"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.CommentsThread = void 0;
const tslib_1 = require("tslib");
const mobx_1 = require("mobx");
const di_1 = require("@opensumi/di");
const ide_core_browser_1 = require("@opensumi/ide-core-browser");
const resource_1 = require("@opensumi/ide-core-browser/lib/contextkey/resource");
const ide_editor_1 = require("@opensumi/ide-editor");
const common_1 = require("../common");
const comments_zone_view_1 = require("./comments-zone.view");
let CommentsThread = class CommentsThread extends ide_core_browser_1.Disposable {
    set contextValue(value) {
        this._contextKeyService.createKey('thread', value);
    }
    get contextValue() {
        return this._contextKeyService.getContextValue('thread');
    }
    get onDidChange() {
        return this.onDidChangeEmitter.event;
    }
    constructor(uri, range, providerId, options) {
        super();
        this.uri = uri;
        this.range = range;
        this.providerId = providerId;
        this.options = options;
        this.comments = [];
        this._readOnly = false;
        this.widgets = new Map();
        this._id = `thread_${(0, ide_core_browser_1.uuid)()}`;
        this.onDidChangeEmitter = new ide_core_browser_1.Emitter();
        this.toggle = (editor) => {
            if (this.comments.length > 0) {
                const widget = this.widgets.get(editor);
                if (widget) {
                    widget.toggle();
                }
            }
            else {
                this.dispose();
            }
        };
        (0, mobx_1.makeObservable)(this);
        this.updateComments(options.comments
            ? options.comments.map((comment) => (Object.assign(Object.assign({}, comment), { id: (0, ide_core_browser_1.uuid)() })))
            : []);
        this.data = this.options.data;
        this._contextKeyService = this.registerDispose(this.globalContextKeyService.createScoped());
        // 设置 resource context key
        const resourceContext = new resource_1.ResourceContextKey(this._contextKeyService);
        resourceContext.set(uri);
        this._contextKeyService.createKey('thread', options.contextValue);
        this.readOnly = !!options.readOnly;
        this.label = options.label;
        this.isCollapsed = !!this.options.isCollapsed;
        const threadsLengthContext = this._contextKeyService.createKey('threadsLength', this.commentsService.getThreadsByUri(uri).length);
        const commentsLengthContext = this._contextKeyService.createKey('commentsLength', this.comments.length);
        // vscode 用于判断 thread 是否为空
        const commentThreadIsEmptyContext = this._contextKeyService.createKey('commentThreadIsEmpty', !this.comments.length);
        // vscode 用于判断是否为当前 controller 注册
        this._contextKeyService.createKey('commentController', providerId);
        // 监听 comments 的变化
        (0, mobx_1.autorun)(() => {
            commentsLengthContext.set(this.comments.length);
            commentThreadIsEmptyContext.set(!this.comments.length);
        });
        (0, mobx_1.autorun)(() => {
            if (this.isCollapsed) {
                this.hideAll();
            }
            else {
                this.showAll();
            }
        });
        // 监听每次 thread 的变化，重新设置 threadsLength
        this.addDispose(this.commentsService.onThreadsChanged((thread) => {
            if (thread.uri.isEqual(uri)) {
                threadsLengthContext.set(this.commentsService.getThreadsByUri(uri).length);
            }
        }));
        this.addDispose({
            dispose: () => {
                this.updateComments([]);
            },
        });
        this.onDidChangeEmitter.fire();
    }
    updateComments(comments) {
        this.comments = comments;
    }
    getWidgetByEditor(editor) {
        return this.widgets.get(editor);
    }
    get id() {
        return this._id;
    }
    get contextKeyService() {
        return this._contextKeyService;
    }
    get readOnly() {
        return this._readOnly;
    }
    set readOnly(readOnly) {
        this._readOnly = readOnly;
        this._contextKeyService.createKey('readOnly', this._readOnly);
    }
    get threadHeaderTitle() {
        if (this.label) {
            return this.label;
        }
        if (this.comments.length) {
            const commentAuthors = new Set(this.comments.map((comment) => `@${comment.author.name}`));
            return `${(0, ide_core_browser_1.localize)('comments.participants')}: ` + [...commentAuthors].join(' ');
        }
        else {
            return (0, ide_core_browser_1.localize)('comments.zone.title');
        }
    }
    getEditorsByUri(uri) {
        return this.editorCollectionService.listEditors().filter((editor) => { var _a; return (_a = editor.currentUri) === null || _a === void 0 ? void 0 : _a.isEqual(uri); });
    }
    addWidgetByEditor(editor) {
        const widget = this.injector.get(comments_zone_view_1.CommentsZoneWidget, [editor, this, { arrowColor: 'var(--peekView-border)' }]);
        // 如果当前 widget 发生高度变化，通知同一个 同一个 editor 的其他 range 相同的 thread 也重新计算一下高度
        this.addDispose(widget.onChangeZoneWidget(() => {
            const threads = this.commentsService.commentsThreads.filter((thread) => this.isEqual(thread));
            // 只需要 resize 当前 thread 之后的 thread
            const currentIndex = threads.findIndex((thread) => thread === this);
            const resizeThreads = threads.slice(currentIndex + 1);
            for (const thread of resizeThreads) {
                if (thread.isShowWidget(editor)) {
                    const widget = thread.getWidgetByEditor(editor);
                    widget === null || widget === void 0 ? void 0 : widget.resize();
                }
            }
        }));
        this.addDispose(widget);
        this.widgets.set(editor, widget);
        editor.onDispose(() => {
            widget.dispose();
            this.widgets.delete(editor);
        });
        return widget;
    }
    show(editor) {
        if (editor) {
            let widget = this.widgets.get(editor);
            // 说明是在新的 group 中打开
            if (!widget) {
                widget = this.addWidgetByEditor(editor);
            }
            widget.show();
        }
        else {
            // 每次都拿所有的有这个 uri 的 editor
            const editors = this.getEditorsByUri(this.uri);
            editors.forEach((editor) => {
                var _a;
                let widget = this.widgets.get(editor);
                // 说明是在新的 group 中打开
                if (!widget) {
                    widget = this.addWidgetByEditor(editor);
                }
                // 如果标记之前是已经展示的 widget，则调用 show 方法
                if ((_a = editor.currentUri) === null || _a === void 0 ? void 0 : _a.isEqual(this.uri)) {
                    widget.show();
                }
            });
        }
    }
    showWidgetsIfShowed() {
        var _a;
        for (const editor of this.getEditorsByUri(this.uri)) {
            let widget = this.widgets.get(editor);
            // 说明是在新的 group 中打开
            if (!widget) {
                widget = this.addWidgetByEditor(editor);
            }
            // 如果标记之前是已经展示的 widget，则调用 show 方法
            if (((_a = editor.currentUri) === null || _a === void 0 ? void 0 : _a.isEqual(this.uri)) && widget.isShow) {
                widget.show();
            }
        }
    }
    hideWidgetsByDispose() {
        var _a;
        for (const [editor, widget] of this.widgets) {
            !((_a = editor.currentUri) === null || _a === void 0 ? void 0 : _a.isEqual(this.uri)) && widget.dispose();
        }
    }
    isShowWidget(editor) {
        if (editor) {
            const widget = this.widgets.get(editor);
            return widget ? widget.isShow : false;
        }
        else {
            for (const [, widget] of this.widgets) {
                return widget.isShow;
            }
            return false;
        }
    }
    hide(editor) {
        if (editor) {
            const widget = this.widgets.get(editor);
            widget === null || widget === void 0 ? void 0 : widget.hide();
        }
        else {
            this.hideAll();
        }
    }
    showAll() {
        for (const [, widget] of this.widgets) {
            widget.show();
        }
    }
    hideAll(isDospose) {
        var _a;
        for (const [editor, widget] of this.widgets) {
            if (isDospose) {
                // 如果 thread 出现在当前 editor 则不隐藏
                !((_a = editor.currentUri) === null || _a === void 0 ? void 0 : _a.isEqual(this.uri)) && widget.dispose();
            }
            else {
                widget.hide();
            }
        }
    }
    addComment(...comments) {
        this.comments.push(...comments.map((comment) => (Object.assign(Object.assign({}, comment), { id: (0, ide_core_browser_1.uuid)() }))));
        this.onDidChangeEmitter.fire();
    }
    removeComment(comment) {
        const index = this.comments.findIndex((c) => c === comment);
        if (index !== -1) {
            this.comments.splice(index, 1);
        }
        this.onDidChangeEmitter.fire();
    }
    isEqual(thread) {
        return thread.uri.isEqual(this.uri) && thread.range.startLineNumber === this.range.startLineNumber;
    }
};
tslib_1.__decorate([
    (0, di_1.Autowired)(common_1.ICommentsService),
    tslib_1.__metadata("design:type", Object)
], CommentsThread.prototype, "commentsService", void 0);
tslib_1.__decorate([
    (0, di_1.Autowired)(ide_core_browser_1.IContextKeyService),
    tslib_1.__metadata("design:type", Object)
], CommentsThread.prototype, "globalContextKeyService", void 0);
tslib_1.__decorate([
    (0, di_1.Autowired)(ide_editor_1.EditorCollectionService),
    tslib_1.__metadata("design:type", ide_editor_1.EditorCollectionService)
], CommentsThread.prototype, "editorCollectionService", void 0);
tslib_1.__decorate([
    (0, di_1.Autowired)(di_1.INJECTOR_TOKEN),
    tslib_1.__metadata("design:type", di_1.Injector)
], CommentsThread.prototype, "injector", void 0);
tslib_1.__decorate([
    mobx_1.observable.shallow,
    tslib_1.__metadata("design:type", Array)
], CommentsThread.prototype, "comments", void 0);
tslib_1.__decorate([
    mobx_1.observable,
    tslib_1.__metadata("design:type", Object)
], CommentsThread.prototype, "label", void 0);
tslib_1.__decorate([
    mobx_1.observable,
    tslib_1.__metadata("design:type", Object)
], CommentsThread.prototype, "_readOnly", void 0);
tslib_1.__decorate([
    mobx_1.observable,
    tslib_1.__metadata("design:type", Boolean)
], CommentsThread.prototype, "isCollapsed", void 0);
tslib_1.__decorate([
    mobx_1.action,
    tslib_1.__metadata("design:type", Function),
    tslib_1.__metadata("design:paramtypes", [Array]),
    tslib_1.__metadata("design:returntype", void 0)
], CommentsThread.prototype, "updateComments", null);
tslib_1.__decorate([
    mobx_1.computed,
    tslib_1.__metadata("design:type", Boolean),
    tslib_1.__metadata("design:paramtypes", [Boolean])
], CommentsThread.prototype, "readOnly", null);
tslib_1.__decorate([
    mobx_1.computed,
    tslib_1.__metadata("design:type", Object),
    tslib_1.__metadata("design:paramtypes", [])
], CommentsThread.prototype, "threadHeaderTitle", null);
CommentsThread = tslib_1.__decorate([
    (0, di_1.Injectable)({ multiple: true }),
    tslib_1.__metadata("design:paramtypes", [ide_core_browser_1.URI, Object, String, Object])
], CommentsThread);
exports.CommentsThread = CommentsThread;
//# sourceMappingURL=comments-thread.js.map